require "ease_engine"
require "./echo_packet"

class BenchmarkApplication < EaseEngine::Application
  def on_start
    super
    
    EaseEngine::Frame.fps = 60
    
    host = ARGV[ 0 ]
    port = ARGV[ 1 ].to_i
    @tcp_num = ARGV[ 2 ].to_i
    @udp_num = ARGV[ 3 ].to_i
    
    @tcp_packet = EchoPacket.new
    @tcp_packet.str = "0123456789ABCDEF" * 32 # 512
#    @tcp_packet.str = "0123456789ABCDEF" * 128 # 2048
#    @tcp_packet.str = "0123456789ABCDEF" * 4096 # 64KB
    
    @udp_packet = EchoPacket.new
    @udp_packet.str = "0123456789ABCDEF" * 32 # 512
    
    @tcp_len = @tcp_packet.str.length
    @udp_len = @udp_packet.str.length
    EE_LOG_DBG.call "tcp_len=#{@tcp_len} udp_len=#{@udp_len}"
    
    @tcp_num.times{|i|
      tcp_socket = EaseEngine::TCPSocket.new( host, port )
#      EE_LOG_DBG.call "tcp_num=#{i + 1}"
      add_connect_socket( tcp_socket )
    }
    
    @udp_num.times{|i|
      udp_socket = EaseEngine::UDPSocket.new
#      EE_LOG_DBG.call "udp_num=#{i + 1}"
      add_socket( udp_socket )
      write_packet( udp_socket, @udp_packet, 0, host, port )
    }
    
    @measure.check
    EE_LOG_DBG.call "host=#{host} port=#{port} tcp_num=#{@tcp_num} udp_num=#{@udp_num} watcher.size=#{@watcher.size} update_usec=#{@measure.update_usec}"
    @measure.start
    
    @is_update = ( 0 < @watcher.size )
    
    @measure_read_socket = EaseEngine::Measure.new
  end
  
  def on_update
    if 1000000 <= @measure.check
      EE_LOG_DBG.call "tcp_num=#{@tcp_num} udp_num=#{@udp_num} watcher.size=#{@watcher.size} update_usec=#{@measure.update_usec} count=#{@measure.count}"
      
      @measure.start
      @is_update = ( 0 < @watcher.size )
    end
  end
  
  def on_end
    super
    
    @measure_read_socket.check
    EE_LOG_DBG.call "update_usec=#{@measure_read_socket.update_usec}"
  end
  
  def on_connected_socket( socket )
    write_packet( socket, @tcp_packet, 0 )
  end
  
  def on_read_socket( socket, packet )
    send( "on_#{packet.packet_name.gsub( /\./, '_' )}", socket, packet )
  end
  
  def on_EchoPacket( socket, packet )
    if socket.kind_of?( EaseEngine::TCPSocket )
      @tcp_num -= 1
    else
      @udp_num -= 1
    end
    
    len = packet.str.length
    if @tcp_len != len && @udp_len != len
      EE_LOG_ERR.call "Packet Error len=#{len}"
      @is_update = false
    end
    remove_socket( socket )
    
    @is_update = false if 0 == @watcher.size
  end
end
BenchmarkApplication::run
