require "ease_engine"

class BenchmarkApplication < EaseEngine::Application
  def on_start
    super
    
    EaseEngine::Frame.fps = 60
    
    host = ARGV[ 0 ]
    port = ARGV[ 1 ].to_i
    @tcp_num = ARGV[ 2 ].to_i
    @udp_num = ARGV[ 3 ].to_i
    
    @tcp_num.times{|i|
      tcp_socket = EaseEngine::TCPSocket.new( host, port )
#      EE_LOG_DBG.call "tcp_num=#{i + 1}"
      add_connect_socket( tcp_socket )
    }
    
    measure_packet = EaseEngine::MeasurePacket.new
    
    @udp_num.times{|i|
      udp_socket = EaseEngine::UDPSocket.new
#      EE_LOG_DBG.call "udp_num=#{i + 1}"
      add_socket( udp_socket )
      measure_packet.start
      write_packet( udp_socket, measure_packet, 0, host, port )
    }
    
    @measure.check
    EE_LOG_DBG.call "host=#{host} port=#{port} tcp_num=#{@tcp_num} udp_num=#{@udp_num} watcher.size=#{@watcher.size} update_usec=#{@measure.update_usec}"
    @measure.start
    
    @is_update = ( 0 < @watcher.size )
    
    @measure_read_socket = EaseEngine::Measure.new
  end
  
  def on_update
    if 1000000 <= @measure.check
      EE_LOG_DBG.call "tcp_num=#{@tcp_num} udp_num=#{@udp_num} watcher.size=#{@watcher.size} update_usec=#{@measure.update_usec} count=#{@measure.count}"
      
      @measure.start
      @is_update = ( 0 < @watcher.size )
    end
  end
  
  def on_end
    super
    
    @measure_read_socket.check
    EE_LOG_DBG.call "update_usec=#{@measure_read_socket.update_usec}"
  end
  
  def on_connected_socket( socket )
    write_packet( socket, EaseEngine::MeasurePacket.new, 0 )
  end
  
  def on_read_socket( socket, packet )
    send( "on_#{packet.packet_name.gsub( /\./, '_' )}", socket, packet )
  end
  
  def on_EaseEngine_MeasurePacket( socket, packet )
    if socket.kind_of?( EaseEngine::TCPSocket )
      @tcp_num -= 1
    else
      @udp_num -= 1
    end
    
    EE_LOG_DBG.call "MeasurePacket usec=#{packet.check} #{socket}"
    remove_socket( socket )
    
    @is_update = false if 0 == @watcher.size
  end
end
BenchmarkApplication::run
