require "ease_engine"
require "./udp_packet"

class BenchmarkApplication < EaseEngine::Application
  def on_start
    super
    
    EaseEngine::Frame.fps = 60
    
    host = ARGV[ 0 ]
    port = ARGV[ 1 ].to_i
    udp_num = 1
    
    @base_data_array = [
      0,
      1,
      -1,
      "str",
      "あいう",
      0.0,
      1.5,
      -1.5,
      true,
      false,
      nil
    ]
    
    base_data_packet = BaseDataPacket.new
    base_data_packet.array = @base_data_array
    
    parent_packet = UDP::ParentPacket.new
    parent_packet.parent = 0
    
    child_packet = UDP::ChildPacket.new
    child_packet.child = 1
    
    grandchild_packet = UDP::GrandchildPacket.new
    grandchild_packet.grandchild = 2
    
    udp_num.times{
      socket = EaseEngine::UDPSocket.new
      add_socket( socket )
      write_packet( socket, base_data_packet, 0, host, port )
      write_packet( socket, parent_packet, 0, host, port )
      write_packet( socket, child_packet, 0, host, port )
      write_packet( socket, grandchild_packet, 0, host, port )
    }
    
    @measure.check
    EE_LOG_DBG.call "host=#{host} port=#{port} udp_num=#{udp_num} watcher.size=#{@watcher.size} update_usec=#{@measure.update_usec}"
    @measure.start
    
    @is_update = ( 0 < @watcher.size )
  end
  
  def on_end
    super
    
    @measure.check
    EE_LOG_DBG.call "update_usec=#{@measure.update_usec}"
  end
  
  def on_read_socket( socket, packet )
    send( "on_#{packet.packet_name.gsub( /\./, '_' )}", socket, packet )
  end
  
  def on_BaseDataPacket( socket, packet )
    EE_LOG_DBG.call "#{packet.array}"
    EaseEngine.assert( @base_data_array == packet.array, "#{@base_data_array} != #{packet.array}" )
  end
  
  def on_UDP_ParentPacket( socket, packet )
    EE_LOG_DBG.call "#{packet}"
  end
  
  def on_UDP_ChildPacket( socket, packet )
    EE_LOG_DBG.call "#{packet}"
  end
  
  def on_UDP_GrandchildPacket( socket, packet )
    EE_LOG_DBG.call "#{packet}"
  end
end
BenchmarkApplication::run
