module EaseEngine
  module Data
    class Hary
      class Info
        attr_accessor   :id, :index, :data
        
        def initialize( id, index, data )
          @id = id
          @index = index
          @data = data
        end
      end
      
      def initialize
        @ary = []
        @indexes = {}
      end
      
      def add( id, data )
        if ! @indexes.key?( id )
          info = Info.new( id, @ary.size, data )
          @ary.push info
          @indexes[ id ] = info.index
        else
          info = @ary[ @indexes[ id ] ]
          info.id = id
          info.data = data
        end
        
        sort!
        id
      end
      
      def remove( id )
        remove_at( @indexes[ id ] ) if @indexes.key?( id )
      end
      
      def remove_at( index )
        return nil if @ary.size <= index
        
        info = @ary[ index ]
        @ary.delete_at( index )
        @indexes.delete( info.id )
        update if index != @ary.size # 末尾データの削除なら、更新しない
        info.data
      end
      
      def get( id )
        @indexes[ id ]
      end
      
      def get_at( index )
        ( @ary.size <= index ) ? nil : @ary[ index ]
      end
      
      def key?( id )
        @indexes.key?( id )
      end
      
      def size
        @ary.size
      end
      
      def each
        @ary.each{|info|
          yield info.id, info.index, info.data
        }
      end
      
      def update
        @ary.each_with_index{|info, index|
          info.index = index
          @indexes[ info.id ] = index
        }
      end
      
      def sort!
        update
      end
    end
    
    class Record < Hary
      def initialize( min, max )
        @min = min
        @max = max
        @id = min
        
        super()
      end
      
      def add( data )
        @id = generate_id( @id )
        return 0 if 0 == @id
        
        super( @id, data )
      end
      
    protected
      def generate_id( id )
        generated_id = 0
        start_id = id
        begin
          generated_id = id if ! key?( id )
          
          id = EaseEngine::rotate( @min, @max, id, 1 )
        end while 0 == generated_id && start_id != id
        generated_id
      end
    end
    
    class Group
      def initialize
        @group = {}
        @data = {}
      end
      
      def add( group_id, data_id )
        if @group.key?( group_id )
          @group[ group_id ][ data_id ] = group_id
        else
          @group[ group_id ] = {
            data_id => group_id
          }
        end
        
        if @data.key?( data_id )
          @data[ data_id ][ group_id ] = data_id
        else
          @data[ data_id ] = {
            group_id => data_id
          }
        end
      end
      
      def remove( group_id, data_id )
        if @group.key?( group_id )
          @group[ group_id ].each{|data_id, group_id|
            @data.delete( data_id )
          }
          @group.delete( group_id )
        elsif @data.key?( data_id )
          @data[ data_id ].each{|group_id, data_id|
            @group[ group_id ].delete( data_id )
          }
          @data.delete( data_id )
        end
      end
      
      def group( group_id )
        @group.key?( group_id ) ? @group[ group_id ] : {}
      end
      
      def data( data_id )
        @data.key?( data_id ) ? @data[ data_id ] : {}
      end
    end
  end
end
