require "ease_engine/data"

module EaseEngine
  class Timer
    class Info
      attr_accessor :end_time_usec, :arg, :callback
      
      def initialize( end_time_usec, arg, callback )
        @end_time_usec = end_time_usec
        @arg = arg
        @callback = callback
      end
    end
    
    class Record < EaseEngine::Data::Record
      def sort!
        @ary.sort_by!{|info| info.data.end_time_usec}
        
        super
      end
    end
    
    attr_accessor   :update_time_usec
    
    def initialize( options = {} )
      options[ :update_time_usec ] = 100000 if ! options.key?( :update_time_usec )
      
      @update_time_usec = options[ :update_time_usec ]
      @check_time_usec = 0
      
      @timer = Record.new( 1, 0xFFFFFFFF )
      @group = EaseEngine::Data::Group.new
    end
    
    def add( group_id, timeout_usec, arg, callback )
      timer_id = @timer.add( Info.new( EaseEngine::Time.new.to_unix_epoch_time_usec + timeout_usec, arg, callback ) )
      return 0 if 0 == timer_id
      
      @group.add( group_id, timer_id ) if 0 < group_id
      
      timer_id
    end
    
    def remove( group_id, timer_id = 0 )
      @group.group( group_id ).each{|timer_id, group_id|
        @timer.remove( timer_id )
      }
      @group.remove( group_id, timer_id )
      @timer.remove( timer_id )
    end
    
    def update
      return if 0 == @timer.size
      
      usec = EaseEngine::Time.new.to_unix_epoch_time_usec
      return if usec < @check_time_usec
      
      @timer.each{|id, index, info|
        if info.end_time_usec < usec
          remove( 0, id )
          
          info.callback.call( info.arg )
        else
          # 終了時間の早い順に昇順ソートしているので、後のデータをチェックする必要なし
          break
        end
      }
      
      @check_time_usec = usec + @update_time_usec
    end
  end
end
