require "cool.io"
require "ease_engine/platform"

module EaseEngine
  class Watcher
    class Info < Cool.io::IOWatcher
      attr_accessor     :io, :callbacks
      
      def initialize( io, flags, callbacks )
        @io = io
        @callbacks = callbacks
        
        super( io.kind_of?( EaseEngine::Socket ) ? io.socket : io, flags )
      end
      
      def on_readable
        @callbacks[ :on_read ].call( @io ) if @callbacks.key?( :on_read )
      end
      
      def on_writable
        @callbacks[ :on_write ].call( @io ) if @callbacks.key?( :on_write )
      end
      
      def on_remove
        @callbacks[ :on_remove ].call( @io ) if @callbacks.key?( :on_remove )
      end
    end
    
    def initialize( options = {} )
      # 環境に合わせて適切なものを選択
      if ! options.key?( :backend )
        backend = :select
        if EaseEngine::Platform.mac?
          backend = :kqueue
        elsif EaseEngine::Platform.linux?
          backend = :epoll
        end
        
        options[ :backend ] = backend
      end
      
      EaseEngine::Log.inf( "Watcher #{options}" )
      @loop = Cool.io::Loop.new( options )
      @watches = {}
    end
    
    def watch( timeout = 0 )
      if ! @watches.empty?
        @loop.run_once( timeout )
      elsif 0 < timeout
        sleep timeout
      end
    end
    
    def add( io, callbacks )
      id = io.to_i
      if @watches.key?( id )
        info = @watches[ id ]
        @watches.delete( id )
        info.detach
      end
      
      flags = ""
      flags = "r" if callbacks.key?( :on_read )
      flags = "#{flags}w" if callbacks.key?( :on_write )
      
      @watches[ id ] = Info.new( io, flags, callbacks )
      @watches[ id ].attach @loop
    end
    
    def remove( io )
      id = io.to_i
      return if ! @watches.key?( id )
      
      info = @watches[ id ]
      @watches.delete( id )
      info.detach
      info.on_remove
    end
    
    def size
      @watches.size
    end
    
    def each
      @watches.each{|id, info|
        yield info
      }
    end
    
    def []( id )
      @watches.key?( id ) ? @watches[ id ].io : nil
    end
  end
end
